#-----------------------------------------------------------------------------
# Name:        wxentries.py
# Purpose:     widgets for customized input
#
# Author:      Julio Jimnez Borreguero - Jujibo Software - Spain
#
# Created:     2002/23/08
# RCS-ID:      $Id: wxentries.py v 1.0rc0$
# Copyright:   (c) 2002 Julio Jimnez Borreguero
# Licence:     GPL 
#-----------------------------------------------------------------------------


from wxPython.wx import *
import fpformat


#----------------------------------------------------------------------
class wxNumericEntry(wxTextCtrl):
    def __init__(self, parent, id, *args, **kwargs):
        
        self.__digits = '0123456789-.'
        self.__nSeparator = '.'
        self.__dSeparator = ','
        self.__nDecim = 0
        self.__iniStr = ''
        self.__endStr = ''
        self.__rightAlign = false
        self.__min = None
        self.__max = None
        self.__showHelp = true
        
        if kwargs.has_key('rAlign'):
            self.__rightAlign = kwargs['rAlign']
            del kwargs['rAlign']
        
        if kwargs.has_key('rangHelp'):
            self.__showHelp = kwargs['rangHelp']
            del kwargs['rangHelp']
        
        if kwargs.has_key('iniStr'):
            self.__iniStr = kwargs['iniStr']
            del kwargs['iniStr']
            
        if kwargs.has_key('endStr'):
            self.__endStr = kwargs['endStr']
            del kwargs['endStr']
            
        if kwargs.has_key('numDec'):
            self.__nDecim = kwargs['numDec']
            del kwargs['numDec']
            
        if kwargs.has_key('thSep'):
            self.__nSeparator = kwargs['thSep']
            del kwargs['thSep']
            
        if kwargs.has_key('decSep'):
            self.__dSeparator = kwargs['decSep']
            del kwargs['decSep']
        
        if kwargs.has_key('min'):
            self.__min = kwargs['min']
            del kwargs['min']
        
        if kwargs.has_key('max'):
            self.__max = kwargs['max']
            del kwargs['max']
                    
        if kwargs.has_key('initial'):
            initial = kwargs['initial']
            del kwargs['initial']
        else:
            initial = 0
            
        

        wxTextCtrl.__init__(self, parent, id,   *args, **kwargs)
        
        self.SetValue(initial)
        
        self.__FormatValue()
               
        EVT_CHAR(self, self.onChar)
        EVT_TEXT(self, -1, self.onText)
        EVT_SET_FOCUS(self, self.onFocus)
        EVT_KILL_FOCUS(self, self.onKillFocus)
        EVT_SIZE(self, self.onSize)
        
        
    def onSize(self, event):
        self.__FormatValue()
        event.Skip()
        
    def onFocus(self, event):
        self.__ValueToText()
        self.SetSelection(0, len(wxTextCtrl.GetValue(self)))
        event.Skip()

        
    def onKillFocus(self, event):
        self.__FormatValue()
        event.Skip()
        


    def onChar(self, event):
        key = event.KeyCode()
        self.__oldtext = wxTextCtrl.GetValue(self)
        
        if key < WXK_SPACE or key == WXK_DELETE or key > 255:
            event.Skip()
            return
        
        if chr(key) == '.' and self.__nDecim == 0:
            wxBell()
            return
        
        if chr(key) == '.' and  '.' in self.__oldtext:
            wxBell()
            return

        if chr(key) in self.__digits :
            event.Skip()
            return
        wxBell()

    def onText(self, event):
        
        if self.__focused:
            return
        
        if event.GetString() <> '' and event.GetString() <> '-':
            if self.__IsValid(event.GetString()) :
                self.__value = float(event.GetString())
            else:
                self.__ValueToText()
                wxBell()
        
        event.Skip()
        
    def __fmtRS(self, value):
        dc = wxClientDC(self)
        dc.SetFont(self.GetFont())
        
        
        text = value
        textwidth = dc.GetTextExtent(text)[0]
        dcwidth = self.GetClientSize()[0]
        tope = dcwidth-dc.GetTextExtent(text[-1:])[0]
        
        while textwidth <= tope:
            textwidth = dc.GetTextExtent(text)[0]
            text = ' '+text
            
             
        del dc
        return text[2:]
            
        
        

    def GetValue(self):
        
        if self.__nDecim > 0:
            return float(fpformat.fix(self.__value, self.__nDecim))
        else:
            return int(self.__value)
        
    def GetTextValue(self):
        return fpformat.fix(self.__value, self.__nDecim)
        
    def SetValue(self, value):
        # comprobamos que el valor es vlido
        if self.__min <> None and value < self.__min:
            self.__value = self.__min
            
        elif self.__max <> None and value > self.__max:
            self.__value = self.__max
            
        else:
            self.__value = value
            
        # ahora debemos llamar a la funcin __FormatValue
        self.__FormatValue()
            
        
    
    def GetMin(self):
        return self.__min
    
    def GetMax(self):
        return self.__max
    
    def SetRange(self, min, max):
        self.__min = min
        self.__max = max
    
    def __ValueToText(self):
        sfm = '%.'+str(self.__nDecim)+'f'
        wxTextCtrl.SetValue(self, (sfm % self.__value))
        
    def __IsValid(self, value):
        
        try:
            valFloat = float(value)
        except ValueError:
            return false
        
        if (self.__min == None or valFloat >= self.__min) and (self.__max == None or valFloat <= self.__max):
            return true
        else:
            if self.__showHelp:
                wxMessageBox("Min Value: %d\nMax Value: %d" % (self.__min, self.__max), "Out of range", wxOK | wxICON_EXCLAMATION, self)
            return false
        
        
    def __FormatValue(self):        
        intString = fpformat.fix(self.__value, self.__nDecim)
        
        
        if self.__nDecim > 0:
            intString = intString[:-self.__nDecim-1]
            nstr = fpformat.fix(self.__value, self.__nDecim)[-self.__nDecim:]
            decString = self.__dSeparator+nstr
        else :
            decString = ''
        
        if intString[:1] == '-':    # sign from decStrin
            sign = '-'
            intString = intString[1:]
        else :
            sign = ''
            
        # thousands to thList
        thList = []
        
        if len(intString)%3 <> 0:
            thList.append(intString[:len(intString)%3])
            intString = intString[len(intString)%3:]
            
        while len(intString) <> 0:
            thList.append(intString[:3])
            intString = intString[3:]
            
        for x in thList:
            intString = intString + x + self.__nSeparator
            
        intString = sign + intString[:-1]
            
        self.__focused = true
        text = self.__iniStr + intString + decString + self.__endStr
        if self.__rightAlign:
            text = self.__fmtRS(text)
            
        wxTextCtrl.SetValue(self, text)
        self.__focused = false
        
        
        
    

